@@ -110,6 +110,9 @@ class S7CPU(object):
 	def setCycleTimeLimit(self, newLimit):
 		self.cycleTimeLimit = float(newLimit)
 
+	def setRunTimeLimit(self, timeoutSeconds=0.0):
+		self.__runtimeLimit = timeoutSeconds if timeoutSeconds > 0.0 else None
+
 	def __detectMnemonics(self, parseTree):
 		specs = self.getSpecs()
 		if specs.getConfiguredMnemonics() != S7CPUSpecs.MNEMONICS_AUTO:
@@ -246,18 +249,28 @@ class S7CPU(object):
 		return db
 
 	def __translateInstanceDB(self, rawDB):
-		fbStr = "SFB" if rawDB.fb.isSFB else "FB"
+		if rawDB.fb.fbSymbol is None:
+			fbStr = "SFB" if rawDB.fb.isSFB else "FB"
+			fbNumber = rawDB.fb.fbNumber
+			isSFB = rawDB.fb.isSFB
+		else:
+			fbStr = '"%s"' % rawDB.fb.fbSymbol
+			fbNumber, sym = self.__resolveBlockName((AwlDataType.TYPE_FB_X,
+								 AwlDataType.TYPE_SFB_X),
+								rawDB.fb.fbSymbol)
+			isSFB = sym.type.type == AwlDataType.TYPE_SFB_X
+
 		try:
-			if rawDB.fb.isSFB:
-				fb = self.sfbs[rawDB.fb.fbNumber]
+			if isSFB:
+				fb = self.sfbs[fbNumber]
 			else:
-				fb = self.fbs[rawDB.fb.fbNumber]
+				fb = self.fbs[fbNumber]
 		except KeyError:
 			raise AwlSimError("Instance DB %d references %s %d, "
 				"but %s %d does not exist." %\
 				(rawDB.index,
-				 fbStr, rawDB.fb.fbNumber,
-				 fbStr, rawDB.fb.fbNumber))
+				 fbStr, fbNumber,
+				 fbStr, fbNumber))
 		db = DB(rawDB.index, fb)
 		interface = fb.interface
 		# Sanity checks
@@ -285,26 +298,28 @@ class S7CPU(object):
 	# Translate classic symbols ("abc")
 	def __resolveClassicSym(self, block, insn, oper):
 		if oper.type == AwlOperator.SYMBOLIC:
-			symbol = self.symbolTable.findByName(oper.value.varName)
+			symbol = self.symbolTable.findOneByName(oper.value.varName)
 			if not symbol:
 				raise AwlSimError("Symbol \"%s\" not found in "
 					"symbol table." % oper.value.varName,
 					insn = insn)
-			oper = symbol.operator
+			newOper = symbol.operator.dup()
+			newOper.setInsn(oper.insn)
+			return newOper
 		return oper
 
 	# Translate symbolic OB/FB/FC/DB block name
-	def __resolveBlockName(self, blockTypeId, blockName):
+	def __resolveBlockName(self, blockTypeIds, blockName):
 		if isString(blockName):
-			symbol = self.symbolTable.findByName(blockName)
+			symbol = self.symbolTable.findOneByName(blockName)
 			if not symbol:
 				raise AwlSimError("Symbolic block name \"%s\" "
 					"not found in symbol table." % blockName)
-			if symbol.type.type != blockTypeId:
+			if symbol.type.type not in blockTypeIds:
 				raise AwlSimError("Symbolic block name \"%s\" "
 					"has an invalid type." % blockName)
-			return symbol.operator.value.byteOffset
-		return blockName
+			return symbol.operator.value.byteOffset, symbol
+		return blockName, None
 
 	# Translate local symbols (#abc or P##abc)
 	# If pointer is false, try to resolve #abc.
@@ -322,7 +337,8 @@ class S7CPU(object):
 
 		# Sanity checks
 		if field.dataType.type == AwlDataType.TYPE_ARRAY:
-			if not oper.value.indices:
+			if not oper.value.indices and\
+			   oper.type != AwlOperator.NAMED_LOCAL_PTR:
 				raise AwlSimError("Cannot address array #%s "
 					"without subscript list." %\
 					oper.value.varName)
@@ -359,7 +375,7 @@ class S7CPU(object):
 		assert(oper.value.dbNumber is not None or\
 		       oper.value.dbName is not None)
 		if oper.value.dbNumber is None:
-			symbol = self.symbolTable.findByName(oper.value.dbName)
+			symbol = self.symbolTable.findOneByName(oper.value.dbName)
 			if not symbol:
 				raise AwlSimError("Symbol \"%s\" specified as DB in "
 					"fully qualified operator not found." %\
@@ -477,8 +493,8 @@ class S7CPU(object):
 		self.__detectMnemonics(parseTree)
 		# Translate OBs
 		for obNumber, rawOB in parseTree.obs.items():
-			obNumber = self.__resolveBlockName(AwlDataType.TYPE_OB_X,
-							   obNumber)
+			obNumber, sym = self.__resolveBlockName((AwlDataType.TYPE_OB_X,),
+								obNumber)
 			if obNumber in self.obs:
 				raise AwlSimError("Multiple definitions of "\
 					"OB %d" % obNumber)
@@ -493,8 +509,8 @@ class S7CPU(object):
 			self.obTempPresetHandlers[obNumber] = presetHandlerClass(self)
 		# Translate FBs
 		for fbNumber, rawFB in parseTree.fbs.items():
-			fbNumber = self.__resolveBlockName(AwlDataType.TYPE_FB_X,
-							   fbNumber)
+			fbNumber, sym = self.__resolveBlockName((AwlDataType.TYPE_FB_X,),
+								fbNumber)
 			if fbNumber in self.fbs:
 				raise AwlSimError("Multiple definitions of "\
 					"FB %d" % fbNumber)
@@ -503,8 +519,8 @@ class S7CPU(object):
 			self.fbs[fbNumber] = fb
 		# Translate FCs
 		for fcNumber, rawFC in parseTree.fcs.items():
-			fcNumber = self.__resolveBlockName(AwlDataType.TYPE_FC_X,
-							   fcNumber)
+			fcNumber, sym = self.__resolveBlockName((AwlDataType.TYPE_FC_X,),
+								fcNumber)
 			if fcNumber in self.fcs:
 				raise AwlSimError("Multiple definitions of "\
 					"FC %d" % fcNumber)
@@ -531,8 +547,10 @@ class S7CPU(object):
 
 		# Translate DBs
 		for dbNumber, rawDB in parseTree.dbs.items():
-			dbNumber = self.__resolveBlockName(AwlDataType.TYPE_DB_X,
-							   dbNumber)
+			dbNumber, sym = self.__resolveBlockName((AwlDataType.TYPE_DB_X,
+								 AwlDataType.TYPE_FB_X,
+								 AwlDataType.TYPE_SFB_X),
+								dbNumber)
 			if dbNumber in self.dbs:
 				raise AwlSimError("Multiple definitions of "\
 					"DB %d" % dbNumber)
@@ -597,6 +615,8 @@ class S7CPU(object):
 		self.setMcrActive(False)
 		self.mcrStack = [ ]
 		self.statusWord = S7StatusWord()
+		self.__clockMemByteOffset = None
+		self.setRunTimeLimit()
 
 		self.relativeJump = 1
 
@@ -609,7 +629,7 @@ class S7CPU(object):
 		self.minCycleTime = 86400.0
 		self.maxCycleTime = 0.0
 		self.avgCycleTime = 0.0
-		self.startupTime = 0.0
+		self.startupTime = self.__getTime()
 		self.__speedMeasureStartTime = 0
 		self.__speedMeasureStartInsnCount = 0
 		self.__speedMeasureStartCycleCount = 0
@@ -693,6 +713,14 @@ class S7CPU(object):
 		self.__speedMeasureStartCycleCount = 0
 		self.startupTime = self.now
 
+		if self.specs.clockMemByte >= 0:
+			self.__clockMemByteOffset = AwlOffset(self.specs.clockMemByte)
+		else:
+			self.__clockMemByteOffset = None
+		self.__nextClockMemTime = self.now + 0.05
+		self.__clockMemCount = 0
+		self.__clockMemCountLCM = math_lcm(2, 4, 5, 8, 10, 16, 20)
+
 		# Run startup OB
 		if 102 in self.obs and self.is4accu:
 			# Cold start.
@@ -742,19 +770,49 @@ class S7CPU(object):
 			self.__speedMeasureStartInsnCount = self.__insnCount
 			self.__speedMeasureStartCycleCount = self.__cycleCount
 
-	def __updateTimestamp_perf(self):
-		self.now = time.perf_counter()
+	__getTime = getattr(time, "perf_counter", time.time)
 
-	def __updateTimestamp_time(self):
-		self.now = time.time()
-
-	# Construct updateTimestamp() method.
 	# updateTimestamp() updates self.now, which is a
 	# floating point count of seconds.
-	if hasattr(time, "perf_counter"):
-		updateTimestamp = __updateTimestamp_perf
-	else:
-		updateTimestamp = __updateTimestamp_time
+	def updateTimestamp(self):
+		# Update the system time
+		self.now = self.__getTime()
+		# Update the clock memory byte
+		if self.__clockMemByteOffset:
+			try:
+				if self.now >= self.__nextClockMemTime:
+					self.__nextClockMemTime += 0.05
+					value = self.flags.fetch(self.__clockMemByteOffset, 8)
+					value ^= 0x01 # 0.1s period
+					count = self.__clockMemCount + 1
+					self.__clockMemCount = count
+					if not count % 2:
+						value ^= 0x02 # 0.2s period
+					if not count % 4:
+						value ^= 0x04 # 0.4s period
+					if not count % 5:
+						value ^= 0x08 # 0.5s period
+					if not count % 8:
+						value ^= 0x10 # 0.8s period
+					if not count % 10:
+						value ^= 0x20 # 1.0s period
+					if not count % 16:
+						value ^= 0x40 # 1.6s period
+					if not count % 20:
+						value ^= 0x80 # 2.0s period
+					if count >= self.__clockMemCountLCM:
+						self.__clockMemCount = 0
+					self.flags.store(self.__clockMemByteOffset, 8, value)
+			except AwlSimError as e:
+				raise AwlSimError("Failed to generate clock "
+					"memory signal:\n" + str(e) +\
+					"\n\nThe configured clock memory byte "
+					"address might be invalid." )
+		# Check whether the runtime timeout exceeded
+		if self.__runtimeLimit is not None:
+			if self.now - self.startupTime >= self.__runtimeLimit:
+				raise MaintenanceRequest(MaintenanceRequest.TYPE_RTTIMEOUT,
+					"CPU runtime timeout")
 
 	__dateAndTimeWeekdayMap = {
 		0	: 2,	# monday
