@@ -430,10 +430,10 @@ class AwlInsn(object):
 	type2name_english = pivotDict(name2type_english)
 
 	def __init__(self, cpu, type, rawInsn):
+		self.cpu = cpu
 		self.type = type
 		self.rawInsn = rawInsn
-		self.cpu = cpu
-		self.ip = None
+		self.ip = None		# Instruction pointer (IP)
 		self.ops = []		# Operators
 		self.params = []	# Parameter assignments (for CALL)
 
@@ -464,6 +464,11 @@ class AwlInsn(object):
 	def getCpu(self):
 		return self.cpu
 
+	def getSourceId(self):
+		if not self.rawInsn:
+			return None
+		return self.rawInsn.getSourceId()
+
 	def getLineNr(self):
 		if not self.rawInsn:
 			return -1
