@@ -202,24 +202,26 @@ class AwlSimMessage_RUNSTATE(AwlSimMessage):
 		return cls(runState)
 
 class AwlSimMessage_EXCEPTION(AwlSimMessage):
-	def __init__(self, exceptionText):
+	def __init__(self, exception):
 		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_EXCEPTION)
-		self.exceptionText = exceptionText
+		self.exception = exception
 
 	def toBytes(self):
 		try:
-			textBytes = self.exceptionText.encode()
-			return AwlSimMessage.toBytes(self, len(textBytes)) + textBytes
-		except UnicodeError:
-			raise TransferError("EXCEPTION: Unicode error")
+			pl = self.packString(self.exception.getReport(verbose = False)) +\
+			     self.packString(self.exception.getReport(verbose = True))
+			return AwlSimMessage.toBytes(self, len(pl)) + pl
+		except ValueError:
+			raise TransferError("EXCEPTION: Encoding error")
 
 	@classmethod
 	def fromBytes(cls, payload):
 		try:
-			text = payload.decode()
-		except UnicodeError:
-			raise TransferError("EXCEPTION: Unicode error")
-		return cls(text)
+			text, count = cls.unpackString(payload)
+			verboseText, count = cls.unpackString(payload, count)
+		except ValueError:
+			raise TransferError("EXCEPTION: Encoding error")
+		return cls(AwlSimErrorText(text, verboseText))
 
 class _AwlSimMessage_source(AwlSimMessage):
 	sourceClass = None
@@ -372,21 +374,23 @@ class AwlSimMessage_CPUDUMP(AwlSimMessage):
 class AwlSimMessage_MAINTREQ(AwlSimMessage):
 	plStruct = struct.Struct(str(">H"))
 
-	def __init__(self, requestType):
+	def __init__(self, maintRequest):
 		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_MAINTREQ)
-		self.requestType = requestType
+		self.maintRequest = maintRequest
 
 	def toBytes(self):
-		pl = self.plStruct.pack(self.requestType)
+		pl = self.plStruct.pack(self.maintRequest.requestType) +\
+		     self.packString(str(self.maintRequest))
 		return AwlSimMessage.toBytes(self, len(pl)) + pl
 
 	@classmethod
 	def fromBytes(cls, payload):
 		try:
-			(requestType, ) = cls.plStruct.unpack(payload)
-		except struct.error as e:
+			(requestType, ) = cls.plStruct.unpack_from(payload, 0)
+			msg, count = cls.unpackString(payload, cls.plStruct.size)
+		except (struct.error, ValueError) as e:
 			raise TransferError("MAINTREQ: Invalid data format")
-		return cls(requestType)
+		return cls(MaintenanceRequest(requestType, msg))
 
 class AwlSimMessage_GET_CPUSPECS(AwlSimMessage):
 	def __init__(self):
@@ -408,7 +412,8 @@ class AwlSimMessage_CPUSPECS(AwlSimMessage):
 					self.cpuspecs.nrInputs,
 					self.cpuspecs.nrOutputs,
 					self.cpuspecs.nrLocalbytes,
-					*( (0,) * 24 ) # padding
+					self.cpuspecs.clockMemByte & 0xFFFFFFFF,
+					*( (0,) * 23 ) # padding
 		)
 		return AwlSimMessage.toBytes(self, len(pl)) + pl
 
@@ -418,7 +423,7 @@ class AwlSimMessage_CPUSPECS(AwlSimMessage):
 			data = cls.plStruct.unpack(payload)
 			(mnemonics, nrAccus, nrTimers,
 			 nrCounters, nrFlags, nrInputs,
-			 nrOutputs, nrLocalbytes) = data[:8]
+			 nrOutputs, nrLocalbytes, clockMemByte) = data[:9]
 		except struct.error as e:
 			raise TransferError("CPUSPECS: Invalid data format")
 		cpuspecs = S7CPUSpecs()
@@ -430,6 +435,7 @@ class AwlSimMessage_CPUSPECS(AwlSimMessage):
 		cpuspecs.setNrInputs(nrInputs)
 		cpuspecs.setNrOutputs(nrOutputs)
 		cpuspecs.setNrLocalbytes(nrLocalbytes)
+		cpuspecs.setClockMemByte(-1 if clockMemByte > 0xFFFF else clockMemByte)
 		return cls(cpuspecs)
 
 class AwlSimMessage_REQ_MEMORY(AwlSimMessage):
