@@ -22,12 +22,60 @@
 from __future__ import division, absolute_import, print_function, unicode_literals
 from awlsim.core.compat import *
 
-from awlsim.core.templates import *
-
 from awlsim.gui.util import *
 from awlsim.gui.sourcetabs import *
 
 
+class TemplateDialog(QDialog):
+	def __init__(self, blockName, verboseBlockName=None, extra=None, parent=None):
+		QDialog.__init__(self, parent)
+		self.setLayout(QGridLayout())
+
+		if not verboseBlockName:
+			verboseBlockName = blockName
+
+		self.setWindowTitle("Insert %s template" % verboseBlockName)
+
+		label = QLabel("Insert %s template." % verboseBlockName, self)
+		self.layout().addWidget(label, 0, 0, 1, 2)
+
+		label = QLabel("%s number:" % verboseBlockName, self)
+		self.layout().addWidget(label, 1, 0)
+		self.blockNr = QSpinBox(self)
+		self.blockNr.setMinimum(1)
+		self.blockNr.setMaximum(0xFFFF)
+		self.blockNr.setValue(1)
+		self.blockNr.setPrefix(blockName + " ")
+		self.layout().addWidget(self.blockNr, 1, 1)
+
+		if extra:
+			label = QLabel("%s number:" % extra, self)
+			self.layout().addWidget(label, 2, 0)
+			self.extraNr = QSpinBox(self)
+			self.extraNr.setMinimum(1)
+			self.extraNr.setMaximum(0xFFFF)
+			self.extraNr.setValue(1)
+			self.extraNr.setPrefix(extra + " ")
+			self.layout().addWidget(self.extraNr, 2, 1)
+
+		self.verbose = QCheckBox("Generate verbose code", self)
+		self.verbose.setCheckState(Qt.Checked)
+		self.layout().addWidget(self.verbose, 3, 0, 1, 2)
+
+		self.okButton = QPushButton("&Paste code", self)
+		self.layout().addWidget(self.okButton, 4, 0, 1, 2)
+
+		self.okButton.released.connect(self.accept)
+
+	def getBlockNumber(self):
+		return self.blockNr.value()
+
+	def getExtraNumber(self):
+		return self.extraNr.value()
+
+	def getVerbose(self):
+		return self.verbose.checkState() == Qt.Checked
+
 class ProjectWidget(QTabWidget):
 	# Signal: Some source changed
 	codeChanged = Signal()
@@ -42,6 +90,7 @@ class ProjectWidget(QTabWidget):
 
 		self.__project = Project(None) # Empty project
 		self.__isAdHocProject = False
+		self.__warnedFileBacked = False
 
 		self.awlTabs = AwlSourceTabWidget(self)
 		self.symTabs = SymSourceTabWidget(self)
@@ -82,6 +131,7 @@ class ProjectWidget(QTabWidget):
 		self.__project = project
 		self.awlTabs.setSources(self.__project.getAwlSources())
 		self.symTabs.setSources(self.__project.getSymTabSources())
+		self.__warnedFileBacked = False
 
 	def __loadPlainAwlSource(self, filename):
 		project = Project(None) # Create an ad-hoc project
@@ -132,8 +182,9 @@ class ProjectWidget(QTabWidget):
 		if not all(awlSrcs) or not all(symTabSrcs):
 			# Failed to generate some sources
 			return 0
-		if any(src.isFileBacked() for src in awlSrcs) or\
-		   any(src.isFileBacked() for src in symTabSrcs):
+		if (any(src.isFileBacked() for src in awlSrcs) or\
+		    any(src.isFileBacked() for src in symTabSrcs)) and\
+		    not self.__warnedFileBacked:
 			QMessageBox.information(self,
 				"Project contains external sources",
 				"The project contains external sources.\n"
@@ -141,6 +192,7 @@ class ProjectWidget(QTabWidget):
 				"external sources into the project.\n"
 				"Click on 'integrate source into project' "
 				"in the source menu.")
+			self.__warnedFileBacked = True
 		self.__project.setAwlSources(awlSrcs)
 		self.__project.setSymTabSources(symTabSrcs)
 		self.__project.setProjectFile(filename)
@@ -164,22 +216,45 @@ class ProjectWidget(QTabWidget):
 				"the template to.")
 
 	def insertOB(self):
-		self.__pasteAwlText(awlTemplate_OB)
+		dlg = TemplateDialog("OB", parent=self)
+		if dlg.exec_() == QDialog.Accepted:
+			self.__pasteAwlText(Templates.getOB(dlg.getBlockNumber(),
+							    dlg.getVerbose()))
 
 	def insertFC(self):
-		self.__pasteAwlText(awlTemplate_FC)
+		dlg = TemplateDialog("FC", parent=self)
+		if dlg.exec_() == QDialog.Accepted:
+			self.__pasteAwlText(Templates.getFC(dlg.getBlockNumber(),
+							    dlg.getVerbose()))
 
 	def insertFB(self):
-		self.__pasteAwlText(awlTemplate_FB)
+		dlg = TemplateDialog("FB", parent=self)
+		if dlg.exec_() == QDialog.Accepted:
+			self.__pasteAwlText(Templates.getFB(dlg.getBlockNumber(),
+							    dlg.getVerbose()))
 
 	def insertInstanceDB(self):
-		self.__pasteAwlText(awlTemplate_instanceDB)
+		dlg = TemplateDialog("DB", "Instance-DB", extra="FB", parent=self)
+		if dlg.exec_() == QDialog.Accepted:
+			self.__pasteAwlText(Templates.getInstanceDB(dlg.getBlockNumber(),
+								    dlg.getExtraNumber(),
+								    dlg.getVerbose()))
 
 	def insertGlobalDB(self):
-		self.__pasteAwlText(awlTemplate_globalDB)
+		dlg = TemplateDialog("DB", parent=self)
+		if dlg.exec_() == QDialog.Accepted:
+			self.__pasteAwlText(Templates.getGlobalDB(dlg.getBlockNumber(),
+								  dlg.getVerbose()))
 
 	def insertFCcall(self):
-		self.__pasteAwlText(awlTemplate_FCcall)
+		dlg = TemplateDialog("FC", "FC call", parent=self)
+		if dlg.exec_() == QDialog.Accepted:
+			self.__pasteAwlText(Templates.getFCcall(dlg.getBlockNumber(),
+								dlg.getVerbose()))
 
 	def insertFBcall(self):
-		self.__pasteAwlText(awlTemplate_FBcall)
+		dlg = TemplateDialog("FB", "FB call", extra="DB", parent=self)
+		if dlg.exec_() == QDialog.Accepted:
+			self.__pasteAwlText(Templates.getFBcall(dlg.getBlockNumber(),
+								dlg.getExtraNumber(),
+								dlg.getVerbose()))
