@@ -49,7 +49,7 @@ get_interpreter_version()
 	"$interpreter" -c 'import sys; print("%d %d %d" % sys.version_info[0:3]);' 2>/dev/null
 }
 
-# $1=interpreter $2=awl_file ($3ff additional options to awlsimcli)
+# $1=interpreter $2=awl_file ($3ff additional options to awlsim-cli)
 run_awl_test()
 {
 	local interpreter="$1"
@@ -57,7 +57,7 @@ run_awl_test()
 	shift; shift
 
 	command time -o "$test_time_file" -f '%E' \
-	"$interpreter" "$rootdir/awlsimcli" --quiet --onecycle --extended-insns \
+	"$interpreter" "$rootdir/awlsim-cli" --quiet --extended-insns \
 		--hardware debug:inputAddressBase=7:outputAddressBase=8:dummyParam=True \
 		--cycle-time 60 \
 		"$@" \
@@ -91,7 +91,7 @@ run_sh_test()
 	echo "[OK]"
 }
 
-# $1=interpreter $2=testfile(.awl/.sh) ($3ff additional options to awlsimcli or testfile)
+# $1=interpreter $2=testfile(.awl/.sh) ($3ff additional options to awlsim-cli or testfile)
 run_test()
 {
 	local interpreter="$1"
@@ -125,14 +125,25 @@ run_test_directory()
 	for entry in "$directory"/*; do
 		[ -d "$entry" ] && continue
 		[ "$(echo -n "$entry" | tail -c7)" = ".awlpro" ] || continue
-		run_test "$interpreter" "$entry"
+
+		local extra=
+		[ "$(basename "$entry")" = "EXAMPLE.awlpro" -o\
+		  "$(basename $(dirname "$entry"))" = "projects" ] &&\
+			local extra="--max-runtime 1.0"
+
+		run_test "$interpreter" "$entry" $extra
 	done
 	# run .awl tests
 	for entry in "$directory"/*; do
 		[ -d "$entry" ] && continue
 		[ "$(echo -n "$entry" | tail -c4)" = ".awl" ] || continue
 		[ -e "${entry}pro" ] && continue
-		run_test "$interpreter" "$entry"
+
+		local extra=
+		[ "$(basename $(dirname "$entry"))" = "projects" ] &&\
+			local extra="--max-runtime 1.0"
+
+		run_test "$interpreter" "$entry" $extra
 	done
 	# run .sh tests
 	for entry in "$directory"/*; do
